#include <math.h>
#include "MPU9250.h"
#include "spi.h"
double accelRange,gyroRange;

uint8_t accelBuf[6],gyroBuf[6];


#define READ_FLAG   0x80


//start Accelerometer
void MPU9250_beginAccel(uint8_t mode) {
  switch(mode) {
  case ACC_FULL_SCALE_2_G:
    accelRange = 2.0;
    break;
  case ACC_FULL_SCALE_4_G:
    accelRange = 4.0;
    break;
  case ACC_FULL_SCALE_8_G:
    accelRange = 8.0;
    break;
  case ACC_FULL_SCALE_16_G:
    accelRange = 16.0;
    break;
  default:
    return; // Return without writing invalid mode
  }

  uint8_t data[2]={MPU9250_ADDR_ACCELCONFIG,mode};
  cs_low();
  spi_transmit(data, 2);
  cs_high();
	
}

//start gyroscope
void MPU9250_beginGyro(uint8_t mode) {
  
  switch (mode) {
  case GYRO_FULL_SCALE_250_DPS:
    gyroRange = 250.0;
    break;
  case GYRO_FULL_SCALE_500_DPS:
    gyroRange = 500.0;
    break;
  case GYRO_FULL_SCALE_1000_DPS:
    gyroRange = 1000.0;
    break;
  case GYRO_FULL_SCALE_2000_DPS:
    gyroRange = 2000.0;
    break;
  default:
    return; // Return without writing invalid mode
  }
  uint8_t data[2]={27,mode};
  cs_low();
  spi_transmit(data, 2);
  cs_high();
}

//start compass




uint8_t MPU9250_accelUpdate(void)
{
	uint8_t data[6]={0x3B|READ_FLAG,0xFF,0xFF,0xFF,0xFF,0xFF};
	cs_low();
	SPI_DMA_Receive(accelBuf, 6);
	SPI_DMA_Transmit(data, 6);
	while(finished_receive()==RX_Inprogress);
	reset_finished();
	cs_high();
	return 0;
 
}

float MPU9250_accelGet(uint8_t highIndex, uint8_t lowIndex) {
  int16_t v = ((int16_t) accelBuf[highIndex]) << 8 | accelBuf[lowIndex];
  return ((float) -v) * accelRange / (float) 0x8000; // (float) 0x8000 == 32768.0
}

float MPU9250_accelSqrt(void) {
  return sqrt(pow(MPU9250_accelGet(0, 1), 2) +
              pow(MPU9250_accelGet(2, 3), 2) +
              pow(MPU9250_accelGet(4, 5), 2));
}

float MPU9250_accelX(void) {
  return MPU9250_accelGet(0, 1);
}

float MPU9250_accelY(void) {
  return MPU9250_accelGet(2, 3);
}

float MPU9250_accelZ(void) {
  return MPU9250_accelGet(4, 5);
}


uint8_t MPU9250_gyroUpdate(void)
{
	uint8_t data[6]={0x43|READ_FLAG,0xFF,0xFF,0xFF,0xFF,0xFF};
	cs_low();
	SPI_DMA_Receive(gyroBuf,6);
	SPI_DMA_Transmit(data, 6);
	while(finished_receive()==RX_Inprogress);
	cs_high();
	reset_finished();
	return 0;
}

float MPU9250_gyroGet(uint8_t highIndex, uint8_t lowIndex) {
  int16_t v = ((int16_t) gyroBuf[highIndex]) << 8 | gyroBuf[lowIndex];
  return ((float) -v) * gyroRange / (float) 0x8000;
}

float MPU9250_gyroX(void) {
  return MPU9250_gyroGet(0, 1);
}

float MPU9250_gyroY(void) {
  return MPU9250_gyroGet(2, 3);
}

float MPU9250_gyroZ(void) {
  return MPU9250_gyroGet(4, 5);
}





