/*
 * spi.c
 *
 *  Created on: Mar 1, 2023
 *      Author: hussamaldean
 */


#include "spi.h"
#include "stm32f1xx.h"

volatile uint8_t SPI_Transfer_TX_Finished=0;

volatile uint8_t SPI_Transfer_RX_Finished=0;

void spi_DMA_Init(void)
{
	//Enable Port A clock
	RCC->APB2ENR |= RCC_APB2ENR_IOPAEN;
	RCC->APB2ENR |= RCC_APB2ENR_AFIOEN;
	/* PA5-SCK and PA7-MOSI */
	//Mode: Output, Speed: 10MHz
	GPIOA->CRL &= ~(GPIO_CRL_MODE5);
	GPIOA->CRL &= ~(GPIO_CRL_MODE7);
	GPIOA->CRL |= (GPIO_CRL_MODE5_0);
	GPIOA->CRL |= (GPIO_CRL_MODE7_0);
	//Alternate function push-pull
	GPIOA->CRL &= ~(GPIO_CRL_CNF5);
	GPIOA->CRL &= ~(GPIO_CRL_CNF7);
	GPIOA->CRL |= (GPIO_CRL_CNF5_1);
	GPIOA->CRL |= (GPIO_CRL_CNF7_1);

	/* PA5-MISO */
	//Mode input
	GPIOA->CRL &= ~(GPIO_CRL_MODE6);
	//Floating input
	GPIOA->CRL &= ~(GPIO_CRL_CNF6);
	GPIOA->CRL |= (GPIO_CRL_CNF6_0);

	//Remap 0: PA5, PA6, PA7
	AFIO->MAPR &= ~(1UL << 0);

	/*Configure PA0 as output CS*/
	GPIOA->CRL|=GPIO_CRL_MODE0;
	GPIOA->CRL&=~(GPIO_CRL_CNF0);


	//Enable SPI Clock
	RCC->APB2ENR |= RCC_APB2ENR_SPI1EN;

	//Mode: Master
	SPI1->CR1 |= SPI_CR1_MSTR;
	//Baud rate to (8MHz / 2 = 4MHz)
	SPI1->CR1 &= ~SPI_CR1_BR;
	//MSB first
	SPI1->CR1 &= ~(SPI_CR1_LSBFIRST);
	//Full duplex (Transmit/Receive)
	SPI1->CR1 &= ~(SPI_CR1_RXONLY);
	//Data format 8-bit
	SPI1->CR1 &= ~(SPI_CR1_DFF);
	//Software Slave select
	SPI1->CR1 |= SPI_CR1_SSI;
	SPI1->CR1 |= SPI_CR1_SSM;

	/*Set TXDMA bit in CR2*/
	SPI1->CR2|=SPI_CR2_TXDMAEN;

	/*Set RXDMA bit in CR2*/
	SPI1->CR2|=SPI_CR2_RXDMAEN;


	//SPI Enable
	SPI1->CR1 |= SPI_CR1_SPE;

	//Clear initial flags
	(void)SPI1->SR;


	/*DMA Configuration*/

	/*DMA1_Channel3 is SPI1_TX*/
	RCC->AHBENR|=RCC_AHBENR_DMA1EN;
	/*Set the DMA with the following parameters
	 *
	 * 1. Memory Increment mode
	 * 2. Direction Read from Memory
	 * 3. Transfer Complete Interrupt
	 * */

	DMA1_Channel3->CCR|=DMA_CCR_MINC|DMA_CCR_DIR|DMA_CCR_TCIE;

	/*Set Peripheral Address to be SPI1->DR*/
	DMA1_Channel3->CPAR=(uint32_t)& SPI1->DR;

	/*Enable DMA1_Channel3 interrupt in NVIC*/

	NVIC_EnableIRQ(DMA1_Channel3_IRQn);

	/*DMA1_Channel2 is SPI1_RX*/

	/*Set the DMA with the following parameters
	 *
	 * 1. Memory Increment mode
	 * 2. Direction Read from Peripheral
	 * 3. Transfer Complete Interrupt
	 * */

	DMA1_Channel2->CCR|=DMA_CCR_MINC|DMA_CCR_TCIE;

	/*Set Peripheral Address to be SPI1->DR*/
	DMA1_Channel2->CPAR=(uint32_t)& SPI1->DR;

	/*Enable DMA1_Channel3 interrupt in NVIC*/

	NVIC_EnableIRQ(DMA1_Channel2_IRQn);


}

uint8_t finished_transfer()
{
	return SPI_Transfer_TX_Finished;
}

void reset_finished()
{
	SPI_Transfer_TX_Finished=0;
	SPI_Transfer_RX_Finished=0;
}


uint8_t finished_receive()
{
	return SPI_Transfer_RX_Finished;
}

void SPI_DMA_Transmit(uint8_t *data,uint32_t size)
{
	/*Clear pending flag*/
	DMA1->IFCR = DMA_IFCR_CTCIF3;

	/*Set Peripheral Address to be SPI1->DR*/
	DMA1_Channel3->CPAR=(uint32_t)& SPI1->DR;

	DMA1_Channel3->CMAR=(uint32_t)data;

	DMA1_Channel3->CNDTR=size;

	/*Launch DMA*/
	DMA1_Channel3->CCR |= DMA_CCR_EN;


}


void SPI_DMA_Receive(uint8_t *data,uint32_t size)
{
	/*Clear pending flag*/
	DMA1->IFCR = DMA_IFCR_CTCIF2;

	/*Set Peripheral Address to be SPI1->DR*/
	DMA1_Channel2->CPAR=(uint32_t)& SPI1->DR;

	DMA1_Channel2->CMAR=(uint32_t)data;

	DMA1_Channel2->CNDTR=size;

	/*Launch DMA*/
	DMA1_Channel2->CCR |= DMA_CCR_EN;


}


void cs_low()
{
	GPIOA->BSRR=GPIO_BSRR_BR0;
}

void cs_high()
{
	GPIOA->BSRR=GPIO_BSRR_BS0;
}


void DMA1_Channel3_IRQHandler (void)
{
	/*Check if the source is transfer complete*/

	if((DMA1->ISR & DMA_ISR_TCIF3) == DMA_ISR_TCIF3)
	{
		/*Set flag to 1*/
		SPI_Transfer_TX_Finished=1;

		/*Disable DMA */
		DMA1_Channel3->CCR &= ~DMA_CCR_EN;

		/*Clear pending flag*/
		DMA1->IFCR = DMA_IFCR_CTCIF3;
	}
}


void DMA1_Channel2_IRQHandler (void)
{
	/*Check if the source is transfer complete*/

	if((DMA1->ISR & DMA_ISR_TCIF2) == DMA_ISR_TCIF2)
	{
		/*Set flag to 1*/
		SPI_Transfer_RX_Finished=1;

		/*Disable DMA */
		DMA1_Channel2->CCR &= ~DMA_CCR_EN;

		/*Clear pending flag*/
		DMA1->IFCR = DMA_IFCR_CTCIF2;
	}
}

void spi_transmit(uint8_t *data,uint32_t size)
{
	uint32_t i=0;

	while(i<size)
	{
		 /*Wait for the TXE bit to set in the Status Register*/
		/*This will indicate that the transmit buffer is empty*/
		 while (!((SPI1->SR)&(1<<1))) {};
		   /*Write the data to the Data Register*/
		   SPI1->DR = data[i];
		   i++;
	}
	 /*Wait for the TXE bit to set in the Status Register*/
	 while (!((SPI1->SR)&(1<<1))) {};

	 /*Wait for the BSY bit to reset in the Status Register*/
	 while (((SPI1->SR)&(1<<7))) {};

	 /*Clear OVR flag*/
	(void) SPI1->DR;
	(void) SPI1->SR;
}
